<?php

namespace Drupal\entity_repeat\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\date_recur_modular\Plugin\Field\FieldWidget\DateRecurModularAlphaWidget as parentAlias;

/**
 * Entity Recur widget.
 *
 * @FieldWidget(
 *   id = "entity_repeat",
 *   label = @Translation("Entity Repeat widget"),
 *   field_types = {
 *     "date_recur",
 *   }
 * )
 */
class EntityRepeatWidget extends parentAlias {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $entity = $form_state->getformObject()->getEntity();

    // Check user permission to recur.
    if (!$this->entityRepeatAccess($entity)) {
      return $element;
    }

    $element['entity_repeat_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Replicate this entity for each of the generated dates'),
    ];

    $element['#theme'] = 'entity_repeat_widget';

    return $element;
  }

  /**
   * Checks access to repeat the entity.
   *
   * @param EntityInterface $entity
   * @return bool
   */
  public function entityRepeatAccess(EntityInterface $entity) {
    $access = FALSE;
    $account = \Drupal::currentUser();
    $author = $entity->getOwner();
    $bundle_id = $entity->bundle();

    // User can repeat any content of this type.
    if ($account->hasPermission("repeat any $bundle_id content")) {
      $access = TRUE;
    }

    // User is the author and can repeat their own content.
    if ($account->id() == $author->id() && $account->hasPermission("repeat own $bundle_id content")) {
      $access = TRUE;
    }

    return $access;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateModularWidget(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    parent::validateModularWidget($element, $form_state, $complete_form);

    $fieldname = $element['#parents'][0];
    $entity_repeat_enabled = $form_state->getValue($fieldname)[0]['entity_repeat_enabled'];
    if (!$entity_repeat_enabled) {
      return;
    }

    $entity = $form_state->getformObject()->getEntity();
    $entity->data['entity_repeat_enabled'] = 1;
  }

}
